/*
 *
 *	Counter.h
 *
 *
 */

#ifndef COUNTER_H
#define COUNTER_H

#include <Application.h>
#include <Window.h>
#include <List.h>
#include <String.h>
#include <Resources.h>
#include <time.h>
#include "SettingsFile.h"
#include "NetCountCalendarDatas.h"

class CPreferences;

// for debug
#include <iostream.h>

// signature of application
#define APP_SIGNATURE		"application/x-vnd.mougel-cedric.netcount"
// chemin des preferences
#define CPREF_PATH			"NetCountSetting"
#define CPREF_FILE			"NetCountSetting.inf"
#define NETCOUNT_VERSION	"NetCount v0.8.5"

// connection preferences
#define CPREF_CONN_FILE		"NetCountConnections.inf"

// langage datas
#define CPREF_LANGAGE_FILE	"NetCountLangage.inf"

// messages de sauveardes des prefs de l'appli
#define C_NAME_APP_PREF	"Preferences-Application"

// message de sauvegarde des connexions
#define C_NAME_CONNEXION_PREF	"Preferences-Connexions"

// definition for connections
#define CONNECTION_LIST		"List-Of-Connections"
#define CONNECTION_NUMBER	"NumberConnections"
#define CONNECTION_INDEX	"IndexConnections"
#define CONNECTION_START	"StartConnections"
#define CONNECTION_DURING	"DuringConnections"
#define CONNECTION_PRICE	"PriceConnections"
#define CONNECTION_FIXED	"ConFixedPrice"
// windows frames
#define	APP_WIN_COUNTER		"fenetre-counter"
#define APP_WIN_ABOUT		"fenetre-about"
#define APP_WIN_HISTORY		"fenetre-history"
#define APP_WIN_PRICES		"fenetre-prices"
#define APP_WIN_PREF		"fenetre-pref-app"
// preferences settings
#define APP_VIEW_DESKBAR	"deskbar-view"
#define APP_MINIMIZE_WINDOW	"minimize-window"
#define APP_DISPLAY_TYPE	"display-type"
#define APP_FORCE_RATE		"force-rate"
#define APP_SAVE_AT_END		"save-at-end"
// display preference
#define APP_VIEW_COLOR		"counter-view-color"
#define APP_TEXT_COLOR		"counter-text-color"
#define APP_LANGAGE_TYPE	"langage-type"
// history settings
#define APP_ADD_STATE		"add-state"
#define APP_REMOVE_STATE	"remove-state"
#define APP_FIXED_COLOR		"fixed-price-color"
#define APP_STAND_COLOR		"standard-color"
#define APP_DATE_COLOR		"date-color"
#define APP_TIME_COLOR		"time-color"
#define APP_PRICE_COLOR		"price-color"
// Imports Exports
#define APP_ERASE_STATE		"erase-state"
#define APP_RECALCUL_STATE	"recalcul-time"
#define APP_VERSION_STATE	"version-state"
#define APP_PATH_IMPORT		"import-path"
#define APP_PATH_EXPORT		"export-path"
#define APP_DELIMITER_TYPE	"delimiter-type"
// backup
#define APP_OLDSTART_BACKUP	"oldstart-backup"
#define APP_USE_BACKUP		"use-backup"
#define APP_DELAY_BACKUP	"delay-backup"

// constants for messages and app
const	uint32	U_REFRESH_ALL_COUNT			= 'Urac';
const	uint32	U_MANUAL_STARTSTP_MSG		= 'Umss';


class CounterApp : public BApplication
{
protected:
	/**** hidden settings ****/
	SettingsFile	*_langageData;
	BString			_labelLangage;
	BMessenger		_replicantMessenger;
	float			_totalprice;
	int32			_totaltime;
	int32			_hoursused;
	tm				_usesmonth;	
	int32			_displayLangage;
	bool			_moveWin; 
	/**** user settings ****/
	// Applications
	int32			_viewDeskbar;
	int32			_minimizeAtStart;
	int32			_displayType;
	int32			_pulseRate;
	int32			_saveAtEnd;
	// Display
	rgb_color		_colorView;
	rgb_color		_colorText;
	int32			_langage; // 0=English 1=French 2=Deutch 3=Italian
	// connections window
	int32			_addState;
	int32			_removeState;
	rgb_color		_colorFixed;
	rgb_color		_colorStandard;
	rgb_color		_colorDate;
	rgb_color		_colorTime;
	rgb_color		_colorPrice;
	// Import/Export
	int32			_eraseState;
	int32			_exportState;
	int32			_recalculTime;
	int8			_delimiter;
	BString			_pathImport;
	BString			_pathExport;
	// Backup
	int32			_useBackup;
	int8			_delayBackup;
	tm				_lastBackup;

public:
	BRect					wcounter;
	BRect					wabout;
	BRect					whistory;
	BRect					wprice;
	BRect					wsetting;
	BResources				resApp;
	BList					*connectionlist;
	NetCountCalendarDatas	*priceslist;
	BWindow					*aWindow;
	SettingsFile			*counterpref;
	CPreferences			*counterconnections;
	status_t				prefapp;
	status_t				connects;
	
	CounterApp();
	virtual	~CounterApp();
	virtual void	MessageReceived(BMessage *);
	void		AddDeskbarView();
	void		RemoveDeskbarView();
	void		LoadConnections();
	void		SaveConnections();
	void		CalculTotalTime();	// calcul total time
	void		CalculTotalPrice();	// calcul total price
	status_t	ImportFile(BEntry *file); // import connections
	status_t	ExportFile(BEntry *path,BString *name,int32 at,int32 to); // export connections
	float		ReCalculPrice(tm start,int32 during); // recalcul price for import
	void		FixedMonth(tm &date,bool operation);
	void		LoadSettings();
	void		SaveSettings();
	const char	*Langage(const char *dataName);

	// acces to attributes
	inline int32		ViewDeskbar()			{ return _viewDeskbar; }
	inline int32		MinimizeAtStart()		{ return _minimizeAtStart; }
	inline int32		DisplayType()			{ return _displayType; }
	inline int32		ReplicantRate()			{ return _pulseRate; }
	inline int32		SaveAtEnd()				{ return _saveAtEnd; }
	inline int32		AddState()				{ return _addState; }
	inline int32		RemoveState()			{ return _removeState; }
	inline rgb_color	ColorText()				{ return _colorText; }
	inline rgb_color	ColorView()				{ return _colorView; }
	inline int32		Langage()				{ return _langage; }
	inline rgb_color	ColorFixed()			{ return _colorFixed; }
	inline rgb_color	ColorStandard()			{ return _colorStandard; }
	inline rgb_color	ColorDate()				{ return _colorDate; }
	inline rgb_color	ColorTime()				{ return _colorTime; }
	inline rgb_color	ColorPrice()			{ return _colorPrice; }
	inline int32		TotalTime()				{ return _totaltime; }
	inline float		TotalPrice()			{ return _totalprice; }
	inline BMessenger	*ReplicantMessenger()	{ return &_replicantMessenger; }
	inline int32		HoursUsed()				{ return _hoursused; }
	inline tm			UsesMonth()				{ return _usesmonth; }
	inline int32		EraseState()			{ return _eraseState; }
	inline int32		ExportState()			{ return _exportState; }
	inline int32		RecalculTime()			{ return _recalculTime; }
	inline const char	*PathImport()			{ return _pathImport.String(); }
	inline const char	*PathExport()			{ return _pathExport.String(); }
	inline int8			Delimiter()				{ return _delimiter; }
	inline int32		UseBackup()				{ return _useBackup; }
	inline int8			DelayBackup()			{ return _delayBackup; }
	inline bool			MoveWin()				{ return _moveWin; }
	
	inline void		SetViewDeskbar(int32 value)			{ _viewDeskbar = value; }
	inline void		SetMinimizeAtStart(int32 value)		{ _minimizeAtStart = value; }
	inline void		SetDisplayType(int32 value)			{ _displayType = value; }
	inline void		SetReplicantRate(int32 value)		{ _pulseRate = value; }
	inline void		SetSaveAtEnd(int32 value)			{ _saveAtEnd = value; }
	inline void		SetAddState(int32 value)			{ _addState = value; }
	inline void		SetRemoveState(int32 value)			{ _removeState = value; }
	inline void		SetColorText(rgb_color value)		{ _colorText = value; }
	inline void		SetColorView(rgb_color value)		{ _colorView = value; }
	inline void		SetLangage(int32 value)				{ _langage = value; }
	inline void		SetColorFixed(rgb_color value)		{ _colorFixed = value; }
	inline void		SetColorStandard(rgb_color value)	{ _colorStandard = value; }
	inline void		SetColorDate(rgb_color value)		{ _colorDate = value; }
	inline void		SetColorTime(rgb_color value)		{ _colorTime = value; }
	inline void		SetColorPrice(rgb_color value)		{ _colorPrice = value; }
	inline void		SetHoursUsed(int32 value)			{ _hoursused = value; }
	inline void		SetUsesMonth(tm value)				{ _usesmonth = value; }
	inline void		SetEraseState(int32 value)			{ _eraseState = value; }
	inline void		SetExportState(int32 value)			{ _exportState = value; }
	inline void		SetRecalculTime(int32 value)		{ _recalculTime = value; }
	inline void		SetPathImport(char *value)			{ _pathImport.SetTo(value); }
	inline void		SetPathImport(BString value)		{ _pathImport = value; }
	inline void		SetPathExport(char *value)			{ _pathExport.SetTo(value); }
	inline void		SetPathExport(BString value)		{ _pathExport = value; }
	inline void		SetDelimiter(int8 value)			{ _delimiter = value; }
	inline void		SetUseBackup(int32 value)			{ _useBackup = value; }
	inline void		SetDelayBackup(int8 value)			{ _delayBackup = value; }
	
private:
	void		AddSubMonth(tm &date,bool operation);
	void		AddSubDays(tm &date,bool operation);
	void 		UpdateReplicantTotalTime();
	void		SaveAuto();
	status_t	RestoreLangageData();
};

#endif  // COUNTER_H

