/**************************************************************************
 **
 ** CounterAbout.cpp
 **
 ** The About window with the application's icon
 **
 **************************************************************************
 **
 ** Author  : Nicolas Mougel & Vincent cedric
 ** E-Mail  : mougel@netcourrier.com
 ** E-Mail  : cedric-vincent@wanadoo.fr
 **
 ** If you update this program, please contact the authors.
 **
 **************************************************************************/


#include <Application.h>
#include <Button.h>
#include <Bitmap.h>
#include <Roster.h>
#include <File.h>
#include <AppFileInfo.h>
#include "Counter.h"
#include "CounterAbout.h"
#include "CLaunchNetPositiveView.h"



/* 
 * About window constructor : add a BView
 */

AboutWindow :: AboutWindow(BRect frame,const char *name,BMessenger messenger) :
BWindow(frame,name, B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	_countermessenger = messenger;

	SetLook(B_FLOATING_WINDOW_LOOK);
	SetFeel(B_NORMAL_WINDOW_FEEL);

	AboutView *myView = new AboutView(Bounds());
	AddChild(myView);
}

/**** Quit Requested ****/
bool AboutWindow::QuitRequested(void)
{
	BRect		framefenetre = Frame();
	BMessage	msgtoclose(CLOSE_WIN_ABOUT);
	
	msgtoclose.AddRect("fenetre-about",framefenetre);
	_countermessenger.SendMessage(&msgtoclose);

	return(true);
}

/* 
 * About view constructor : add a BButton
 */

AboutView :: AboutView(BRect rec) :
	BView(rec, "About View", B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	SetViewColor(219,219,219);

	_greatButton=new BButton(BRect(206,123,276,135), "Validate",((CounterApp *)be_app)->Langage("button-great"),new BMessage(B_QUIT_REQUESTED));

	AddChild(new CLaunchNetPositiveView(BRect(95,27,163,39),"c-vincent","Cedric Vincent","http://perso.wanadoo.fr/cvincent/Home_page.html"));
	AddChild(new CLaunchNetPositiveView(BRect(175,27,243,39),"n-mougel","Nicolas Mougel","http://www.labeille.net"));
	AddChild(new CLaunchNetPositiveView(BRect(50,40,128,52),"about-doc",((CounterApp *)be_app)->Langage("ab-doc"),"http://perso.wanadoo.fr/cvincent/logiciels/netcount.html"));
	AddChild(new CLaunchNetPositiveView(BRect(130,40,200,52),"a-meyer","Andreas Meyer","http://home.wtal.de/MeineHomepage"));
	AddChild(_greatButton);
}

/* 
 * About view : to catch the BButton click in this view
 */

void AboutView :: AttachedToWindow()
{
	_greatButton->SetTarget(Window());
}

/* 
 * About view : catch the BButtton message
 */

void AboutView :: MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		default:
			BView::MessageReceived(message);
	}
}

/* 
 * About view : display about text and icon 
 */

void AboutView :: Draw(BRect frame)
{
	rgb_color	oldColor;
	rgb_color	oldLowColor;
	BFont		fontView;

	oldColor = HighColor();
	oldLowColor = LowColor();
	GetFont(&fontView);

	// normal draw for textview
	BView::Draw(frame);
	
	SetLowColor(ViewColor());		
	// Show the About text
	SetFontSize(22.0);
	fontView.SetShear(105.0);
	SetFont(&fontView,B_FONT_SHEAR);
	DrawString(NETCOUNT_VERSION, BPoint(65,22));
	
	SetFontSize(10.0);
	fontView.SetShear(90.0);
	SetFont(&fontView,B_FONT_SHEAR);
	DrawString(((CounterApp *)be_app)->Langage("author"), BPoint(50,37));
	DrawString("&", BPoint(167,37));
	DrawString("& Tiberio G. ",BPoint(205,50));
	DrawString("E-mail :", BPoint(50,67));
	DrawString("cedric-vincent@wanadoo.fr", BPoint(90,67));
	DrawString("mougel@netcourrier.com", BPoint(90,80));
	DrawString("anmeyer@gmx.net", BPoint(90,93));
	DrawString("tiberiog@freemail.it", BPoint(90,106));

	DrawString(((CounterApp *)be_app)->Langage("thank-bull"), BPoint(50,120));

	// Display the icon from the application info
	BFile			file;
	BAppFileInfo	fileinfo;
	app_info		info;

	SetHighColor(189,189,189);
	FillRect(BRect(0,0,30,150));
	
	BBitmap icon(BRect(0,0,31,31 ), B_CMAP8);
	be_app->GetAppInfo(&info);
	file.SetTo(&(info.ref), B_READ_ONLY);
	fileinfo.SetTo(&file);
	fileinfo.GetIcon(&icon, B_LARGE_ICON);
	SetLowColor(ViewColor());
	SetDrawingMode(B_OP_OVER);
	DrawBitmap(&icon, BPoint(10,10));

	SetHighColor(oldColor);
	SetLowColor(oldLowColor);
}