/*
 *
 *	CounterView.cpp
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <Button.h>
#include "Counter.h"
#include "CounterView.h"
#include "HistoryList.h"
#include "PrefAppView.h"
#include "ReplicantCounter.h"
#include "CMenuBarIcon.h"
#include "Constants.h"

extern BApplication *be_app;

#include <iostream.h>
/* CounterView():
 *	Constructor of the counter view
 */

CounterView::CounterView(BRect rect, const char *name)
: BView(rect, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_PULSE_NEEDED)
{
	_app = (CounterApp *)be_app;

	SetFontSize(10.0);

	SetViewColor(_app->ColorView());	
	_pricecolor = ViewColor();

	// Number and state of connection
	_nbconnection = _app->connectionlist->CountItems();
	_connected = false;
	
	// Surf time
	_duringtime = 0;
	_duringprice = 0;
	
    // Start/Stop button
	BRect rec;
	BMessage *plainmsg = new BMessage(U_MANUAL_STARTSTP_MSG);
	rec.Set(115, 18, 140, 35);
	_plainButton = new BButton( rec, "Start", "Start", plainmsg);

	//AddChild(_plainButton);
}

CounterView::~CounterView()
{
}

/* AttachedToWindow() */
/* Attach the button to the window */

void CounterView::AttachedToWindow()
{
	_plainButton->SetTarget(this);

	BView::AttachedToWindow();
}

/* MessageReceived() */
/* Get messages from window */

void CounterView::MessageReceived(BMessage *message)
{
	if (message->WasDropped())
	{
		// handle roColour drag&drop
		const rgb_color *color;
		ssize_t size;
		if (message->FindData("RGBColor", 'RGBC',reinterpret_cast<const void **>(&color), &size) == B_OK)
		{
			BMessage	refresh(U_REFRESH_SETTINGS_MSG);
			_app->SetColorView(*color);
			SetViewColor(*color);
			Invalidate();
			(BMessenger(Window())).SendMessage(&refresh);
		}
	}
	else
	{	
	 	switch ( message->what )
	 	{
	 	/* Start/End Connection for commande line (-start -stop) */
  		case U_MANUAL_STARTSTP_MSG:			
			if (_connected == false)
				StartConnection();	// Connection started		 	
  			else
				EndConnection();	// Connection ended
  			break;					
   		/* refresh total time */
   		case U_REFRESH_ALL_COUNT:
			_nbconnection = _app->connectionlist->CountItems();
   			Invalidate();
   			break;
   		/* Call inherited if we didn't handle the message */
   		default:
	   		BView::MessageReceived(message);
	   		break;
		}
	}
}

/**** ComputeTime() *****/
float CounterView::ComputeTime(tm start,tm stop)
{
	//int32	tabindex;
	double		diffseconde;
	float		price;
	float		total = 0;
	BRect		bounds = Bounds();
	
	if(start.tm_hour != stop.tm_hour)
	{
		tm	hourzero;
		hourzero = stop;
		hourzero.tm_min = 0;
		hourzero.tm_sec = 0;
		diffseconde = difftime(mktime(&hourzero),mktime(&start));
		
		price = _app->priceslist->NetItemPrice(start);
		total = (diffseconde*price);

		start = hourzero;
	}
	diffseconde = difftime(mktime(&stop),mktime(&start));

	_pricecolor = _app->priceslist->NetItemColor(stop);
	Invalidate(BRect(bounds.right-14,25,bounds.right-4,35));
		
	price = _app->priceslist->NetItemPrice(stop);
	total += (diffseconde*price);
	
	return total;
}

/**** Time() *****/
float CounterView::ComputeFixedPrice(tm start,tm stop)
{
	int32	totalsecond;
	tm		temptm;
	double	diffseconde;
	float	price;	
	
	totalsecond = _app->HoursUsed();
	diffseconde = difftime(mktime(&stop),mktime(&start));
	totalsecond += (int32)diffseconde;
	_app->SetHoursUsed(totalsecond);
	
	// out of fixed price
	if(totalsecond > (_app->priceslist->TimeForFixedPrice()*3600))
	{
		if(_app->priceslist->PlusUseCalendar())
			price = ComputeTime(start,stop);
		else
			price = _app->priceslist->PricePlusFixedPrice() * diffseconde;
	}
	else
		price = (_app->priceslist->PriceFixedPrice() / _app->priceslist->TimeForFixedPrice()) *diffseconde;
		
	// finish fixed price and start a new
	temptm = _app->UsesMonth();
	if(difftime(mktime(&stop),mktime(&temptm))>0)
	{
		price = (_app->priceslist->PriceFixedPrice() / _app->priceslist->TimeForFixedPrice()) * diffseconde;
		_app->FixedMonth(temptm,true);
		_app->SetUsesMonth(temptm);
		_app->SetHoursUsed(diffseconde);
	}
	return price;
}

/* StartConnection() */

void CounterView::StartConnection()
{
	time_t		temptime;
	tm			temptm;
	BMessage	message(R_START_CONNECTION_MSG);
	BMessage	bitmap(PPP_CONNECT_MSG);
	
	temptime = time(0);
	_starttime = *localtime(&temptime);
	_stoptime = _starttime;
	_duringprice = 0;
	if(_app->priceslist->UseFixedPrice())
	{
		temptm = _app->UsesMonth();
		if(difftime(temptime,mktime(&temptm))>0)
		{
			_app->FixedMonth(temptm,true);
			_app->SetUsesMonth(temptm);
			_app->SetHoursUsed(0);
		}
	}
	else
	{
		BRect	bounds = Bounds();

		if(_app->priceslist->UseCostConnection())
			_duringprice = _app->priceslist->PriceOfConnection() * 3600;

		_pricecolor = _app->priceslist->NetItemColor(_starttime);
		Invalidate(BRect(bounds.right-14,25,bounds.right-4,35));

	}
	
	_plainButton->SetLabel("Stop");
	message.AddData("start-time",B_ANY_TYPE,&_starttime,sizeof(_starttime));
   	_app->ReplicantMessenger()->SendMessage(&message);
	
	// update bitmap on window
	Window()->PostMessage(&bitmap);
	
	_connected = true;
}

/* EndConnection() */

void CounterView::EndConnection()
{
	time_t		temptime;
 	tm			currenttime;
	BMessage	message(R_END_CONNECTION_MSG);
	BMessage	bitmap(PPP_UNCONNECT_MSG);

	// update bitmap on window
	Window()->PostMessage(&bitmap);

	_connected = false;

	// Get the current time
	temptime = time(0);
	currenttime = *localtime(&temptime);

	// Update surf time
	_duringtime = (int32)difftime(mktime(&currenttime),mktime(&_starttime));

	// Compute price
	if (_app->priceslist->UseFixedPrice() == true)
	{
		// count entire minute if fixed prices is selected
		if (_app->priceslist->EntireMinute() == B_CONTROL_ON)
		{
			int8	deltaSeconde = 0;

			// add second for count entire minute
			deltaSeconde = 60 - (_duringtime % 60 );
			_duringtime += deltaSeconde;
				
			// restore price
			temptime += deltaSeconde;
			currenttime = *localtime(&temptime);
		}
		_duringprice += ComputeFixedPrice(_stoptime,currenttime);
	}
	else
	{
		BRect	bounds = Bounds();

		if(!(_app->priceslist->UseCostConnection() == true && _duringtime < _app->priceslist->TimeForPriceOfConnect()))
			_duringprice += ComputeTime(_stoptime,currenttime);

		_pricecolor = U_COLOR_WHITE;
		Invalidate(BRect(bounds.right-14,25,bounds.right-4,35));
	}
	
	// Add a line of data connection to the list
	_nbconnection++;
	_app->connectionlist->AddItem(new HistoryList(_nbconnection,&_starttime,_duringtime,(_duringprice/3600),DETAILS,_app->priceslist->UseFixedPrice()));

	// update total time and price
   	_app->ReplicantMessenger()->SendMessage(&message);
	_app->CalculTotalTime();
	_app->CalculTotalPrice();

	_plainButton->SetLabel("Start");
	Invalidate();
	
	if(_app->SaveAtEnd() == B_CONTROL_ON)
		_app->SaveConnections();
}


/* Pulse() */
/* Use of pulse to detect Connection / Deconnection of the PPP connection */

void CounterView::Pulse()
{
	thread_info info;
	thread_id	id;

	// Update counter when the PPP connection is on
	if (_connected == true)
	{
		time_t		temptime;
	 	tm			currenttime;
		int32		oldDuringTime;
		
		// Get the current time
		temptime = time(0);
		currenttime = *localtime(&temptime);

		// Update surf time string
		oldDuringTime = _duringtime;
		_duringtime = (int32)difftime(mktime(&currenttime),mktime(&_starttime));

		if(oldDuringTime != _duringtime)
		{
			// Compute price
			if (_app->priceslist->UseFixedPrice() == true)
				_duringprice += ComputeFixedPrice(_stoptime,currenttime);
			else
				if(!(_app->priceslist->UseCostConnection() == true && _duringtime < _app->priceslist->TimeForPriceOfConnect()))
					_duringprice += ComputeTime(_stoptime,currenttime);
			
			_stoptime = currenttime;

			Invalidate(BRect(40,43,152,53));					
			// Connection ended
			if (find_thread("tty_thread")==B_NAME_NOT_FOUND && find_thread("_discon_thread")==B_NAME_NOT_FOUND)
				EndConnection();
		}
	}
	else
	{
		// Connection started
		if (((id=find_thread("tty_thread"))!=B_NAME_NOT_FOUND) && (find_thread("_discon_thread")==B_NAME_NOT_FOUND))
		{
			get_thread_info(id, &info);
			if ((info.state!=B_THREAD_ASLEEP) && (_connected == false))
					StartConnection();
		}
	}
}

/**** drawing the view of count ****/
void CounterView::Draw(BRect frame)
{
	static	char tempstringdraw[30];
	BRect		bounds;
	int32		h,m,s;
	rgb_color	oldColor;
	rgb_color	oldLowColor;

	bounds = Bounds();
	
	oldColor = HighColor();
	oldLowColor = LowColor();
	
	SetHighColor(_app->ColorText());
	SetLowColor(ViewColor());		

	// number of connections
	sprintf(tempstringdraw,"Connections : %02ld",_nbconnection);
	DrawString(tempstringdraw,BPoint(5,35));

	// total surf time and price
	h = _duringtime / 3600;
	m = (_duringtime % 3600) / 60;
	s = _duringtime % 60;

	_tempString.SetTo(_app->Langage("surf-time"));
	sprintf(tempstringdraw,"%02ld:%02ld:%02ld",h,m,s);
	_tempString << tempstringdraw << _app->Langage("for");
	sprintf(tempstringdraw,"%.2f",(_duringprice/3600));
	_tempString << tempstringdraw;
	DrawString(_tempString.String(),BPoint(5,50));

	// total time
	h = _app->TotalTime() / 3600;
	m = (_app->TotalTime() % 3600) / 60;
	s = _app->TotalTime() % 60;

	_tempString.SetTo(_app->Langage("total"));
	if(_app->priceslist->UseFixedPrice())
		_tempString.SetTo(_app->Langage("used"));
		
	sprintf(tempstringdraw," : %02ld:%02ld:%02ld",h,m,s);
	_tempString << tempstringdraw;
	DrawString(_tempString.String(),BPoint(5,65));

	// total price
	_tempString.SetTo(_app->Langage("price"));
	sprintf(tempstringdraw," : %.2f",_app->TotalPrice());
	_tempString << tempstringdraw;
	DrawString(_tempString.String(),BPoint(90,65));
	
	if(!(_app->priceslist->UseFixedPrice()))
	{
		bounds.right -= 3;
		bounds.left = bounds.right -12;
		
		SetHighColor(220,220,220,255);
		StrokeLine(BPoint(bounds.left,24),BPoint(bounds.right,24));
		StrokeLine(BPoint(bounds.left,24),BPoint(bounds.left,36));
		SetHighColor(110,110,110,255);
		StrokeLine(BPoint(bounds.right,24),BPoint(bounds.right,36));
		StrokeLine(BPoint(bounds.left,36),BPoint(bounds.right,36));
		SetHighColor(_pricecolor);
		FillRect(BRect(bounds.left+1,25,bounds.right-1,35));
	}
	
	SetHighColor(oldColor);
	SetLowColor(oldLowColor);
}

