/*
 *
 *	CounterWindow.cpp
 *
 *
 */

#include <MenuBar.h>
#include <MenuItem.h>
#include <Control.h>
#include "CounterWindow.h"
#include "CounterAbout.h"
#include "HistoryWindow.h"
#include "FixPriceWindow.h"
#include "CalendarWinPref.h"
#include "PrefAppWindow.h"
#include "PrefAppView.h"
#include "Counter.h"
#include "CMenuBarIcon.h"


#include <stdio.h>
#include <Alert.h>
/* CounterWindow():
 *	Constructor of the main window
 */

CounterWindow::CounterWindow(BRect frame)
: BWindow(frame,NETCOUNT_VERSION,B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	_app = (CounterApp *)be_app;

	// Set a little window
	SetLook(B_FLOATING_WINDOW_LOOK);
	SetFeel(B_NORMAL_WINDOW_FEEL);

	_historyWindow = NULL;
	_fixPriceWindow = NULL;
	_aboutWindow = NULL;
	_settingWindow = NULL;

	// Build the menu of application
	BuildMenuBar();

	// Set the view of the window
	_aView = new CounterView(Bounds(), "CounterView");
	
	// Add the view to the window
	AddChild(_aView);
	
	// add replicant
	if(_app->ViewDeskbar() == B_CONTROL_ON)	
		_app->AddDeskbarView();
}

void CounterWindow::BuildMenuBar()
{
	// Set the menu bar
	char			shortKey;
	BMenu			*m;
	BRect			r;
	
	r=Bounds();

	// Main menu
	_menuBar = new CMenuBarIcon(r,"Main menu");

	// File Menu	
	m=new BMenu(_app->Langage("file"));
	m->AddItem(new BMenuItem(_app->Langage("about"),new BMessage(U_ABOUT)));
	m->AddSeparatorItem();
	m->AddItem(new BMenuItem(_app->Langage("quit"),new BMessage(B_QUIT_REQUESTED),'Q'));
	_menuBar->AddItem(m);
	
	// History Menu
	m=new BMenu(_app->Langage("option"));
	m->AddItem(new BMenuItem(_app->Langage("view-connection"),new BMessage(U_VIEW_CONNECTIONS),'H'));
	m->AddItem(new BMenuItem(_app->Langage("calendar"),new BMessage(U_FIX_PRICE),'C'));
	m->AddSeparatorItem();
	shortKey = (_app->Langage("setting"))[0];
	m->AddItem(new BMenuItem(_app->Langage("setting"),new BMessage(U_APP_SETTINGS),shortKey));
	_menuBar->AddItem(m);

	AddChild(_menuBar);
}

/* MessageReceived():
 *	Get the messages of the window
 */
 
void CounterWindow::MessageReceived(BMessage *message)
{
	CounterApp		*_app = (CounterApp *)be_app;

	switch(message->what)
	{
		/* About */
		case U_ABOUT:
			OpenAboutWindow();
			break;			
		/* View history details and View history per month */
		case U_VIEW_CONNECTIONS:
			OpenHistoryWindow();
			break;
		/* View Fix Price window */ 
		case U_FIX_PRICE:
			OpenTariffWindow();
			break;
		/* View the Settings Window of Applications */
		case U_APP_SETTINGS:
			OpenSettingsWindow();
			break;
		/* close the about window */		
		case CLOSE_WIN_ABOUT:
			message->FindRect(APP_WIN_ABOUT,&_app->wabout);
			_aboutWindow = NULL;			
			break;
		/* close the history window */
		case CLOSE_WIN_HISTORY:
			message->FindRect(APP_WIN_HISTORY,&_app->whistory);
			_historyWindow = NULL;
			break;
		/* close the tariff window */		
		case CLOSE_WIN_TARIFF:
			message->FindRect(APP_WIN_PRICES,&_app->wprice);
			_fixPriceWindow = NULL;
			break;
		/* close the setting window */
		case CLOSE_WIN_PREF_APP:
			message->FindRect(APP_WIN_PREF,&_app->wsetting);
			_settingWindow = NULL;
			_aView->SetViewColor(_app->ColorView());
			_aView->Invalidate();
			break;
		/* force to refresh datas of setting window	*/
		case U_REFRESH_SETTINGS_MSG:
			if(_settingWindow != NULL)
				(BMessenger(_settingWindow)).SendMessage(message);
			break;
		case U_REFRESH_ALL_COUNT:
			if(_historyWindow != NULL)
				_historyWindow->PostMessage(U_REFRESH_ALL_COUNT);
			(BMessenger(_aView)).SendMessage(message);
			break;
		/* receive message for menuBar */
		case PPP_UNCONNECT_MSG:
		case PPP_CONNECT_MSG:
		case PPP_TRANSMIT_MSG:
			BMessenger(_menuBar).SendMessage(message);
		default:
			BWindow::MessageReceived(message);
	}
}

/* QuitRequested():
 *	Post quit message to the application
 */

bool CounterWindow::QuitRequested()
{
	BAlert 		*alert;

	if (_aView->CounterActive()==true)
	{
		BString	yesMsg(_app->Langage("yes"));
		BString	noMsg(_app->Langage("no"));
		
		alert = new BAlert("Oups",_app->Langage("alert-stop"),yesMsg.String(),noMsg.String(), NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_WARNING_ALERT); 
		if( alert->Go()==1)
        	return(false);
	}

	_app->wcounter = Frame();
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

/**** open the history window ****/
void CounterWindow::OpenHistoryWindow()
{
	if(_historyWindow == NULL)
	{
		_historyWindow = new HistoryWindow(_app->whistory,_app->Langage("history-title"),BMessenger(this));
		_historyWindow->Show();
	}
	else
		_historyWindow->Activate();
}

/**** open the tariff window ****/
void CounterWindow::OpenTariffWindow()
{
	if(_fixPriceWindow == NULL)
	{
		_fixPriceWindow = new FixPriceWindow(_app->wprice,_app->Langage("calendar-title"),BMessenger(this));
		_fixPriceWindow->Show();
	}
	else
		_fixPriceWindow->Activate();
}

/**** open the tariff window ****/
void CounterWindow::OpenAboutWindow()
{
	if(_aboutWindow == NULL)
	{
		_aboutWindow = new AboutWindow(_app->wabout,_app->Langage("about-title"),BMessenger(this));
        _aboutWindow->Show();
	}
	else
		_aboutWindow->Activate();
}

/**** open the tariff window ****/
void CounterWindow::OpenSettingsWindow()
{
	if(_settingWindow == NULL)
	{
		_settingWindow = new PrefAppWindow(_app->wsetting,_app->Langage("app-setting"),BMessenger(this),B_FLOATING_WINDOW_LOOK);
		_settingWindow->Show();
	}
	else
		_settingWindow->Activate();
}

/*** retourne frame setting ***/
BRect CounterWindow::FrameWinSetting()
{
	if(_settingWindow!=NULL)
		return _settingWindow->Frame(); 
	else	
		return _app->wsetting;
}