/***************************/
/* Source of History Views */
/***************************/
#include <ScrollView.h>
#include "HistoryView.h"
#include "CColumnHeader.h"
#include "HistoryWindow.h"
#include "CTabView.h"
#include "Counter.h"


/**** Constructor of history view month-details */
HistoryView::HistoryView(BRect frame,int32 mode,BMessenger messenger)
: BView(frame,"HistoryView",B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	_app = (CounterApp *)be_app;

	// fix color gray to the view
	SetViewColor(219,219,219,255);

	SetFontSize(10.0);
	_history_mode = mode;
	_counterwindow = messenger;

	// Set a little window
	_historyListView = new BListView(BRect(5,25,355,174),"HistoryListView",B_SINGLE_SELECTION_LIST,B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW);
	_historyListView->SetSelectionMessage(new BMessage(U_SELECT_CONNVIEW));
	_historyListView->SetFontSize(11.0);
	AddChild(new BScrollView("scrollHistoryListView",_historyListView,B_FOLLOW_LEFT | B_FOLLOW_TOP,0,false,true));

	// Identyfy the type of drawing	
	CopyListOfConnections();    
}

/**** Destructor of views ****/
HistoryView::~HistoryView()
{
	int32	nbitems;
	
	nbitems = _historyListView->CountItems();
	for(int i=0;i<nbitems;i++)
		delete ((HistoryList*)_historyListView->RemoveItem((int32)0));
}

/****** Drawing the headers of list *****/
void HistoryView::Draw(BRect frame)
{
	DrawHeader(this,BRect(3,6,29,22),"No");
	DrawHeader(this,BRect(30,6,194,22),"Dates");
	DrawHeader(this,BRect(195,6,244,22),_app->Langage("start"));
	DrawHeader(this,BRect(245,6,294,22),_app->Langage("during"));
	DrawHeader(this,BRect(295,6,370,22),_app->Langage("price"));
}

/**** Message received ****/
void HistoryView::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		// Message to set the state of Remove button
		case U_SELECT_CTAB_MSG:
		case U_SELECT_CONNVIEW:
			message->AddBool("list-select",((_historyListView->CurrentSelection() >= 0)?true:false));
			(Window())->PostMessage(message);
			break;
		case U_ADD_HISTORY_ITEM:
			AddNewConnections();
			break;
		// Message to Remove connections
		case U_RM_HISTORY_ITEM:
			RemoveConnections();
			break;
		default:
			BView::MessageReceived(message);
	}
}

/**** attached all controls to this viex ****/
void HistoryView::AttachedToWindow()
{
	_historyListView->SetTarget(this);
	BView::AttachedToWindow();
}

/***** Add Datas to the list for Months drawing mode *****/
void HistoryView::CopyListOfConnections()
{
	int32		nbitems;
	BList		*connectionlist;
	HistoryList	*itemlist;
	tm			start;
	int32		totalduring;
	float		totalcost;
	int32		numconnect;
	
	connectionlist = _app->connectionlist;
	nbitems = connectionlist->CountItems();
	if (nbitems>0)
	{
		if(_history_mode==PER_MONTH)
		{
			numconnect = 0;
			itemlist = (HistoryList *)connectionlist->ItemAt(0);
			start = itemlist->Start();
			totalduring = itemlist->During();
			totalcost = itemlist->Price();
			for(int32 i=1;i<nbitems;i++)
			{
				itemlist = (HistoryList *)connectionlist->ItemAt(i);
				if (start.tm_mon != (itemlist->Start()).tm_mon)
				{
					numconnect++;
					_historyListView->AddItem(new HistoryList(numconnect,&start,totalduring,totalcost,PER_MONTH));
					start = itemlist->Start();
					totalduring = 0;
					totalcost = 0;
				}
				totalduring += itemlist->During();
				totalcost += itemlist->Price();
			}
			numconnect++;
			_historyListView->AddItem(new HistoryList(numconnect,&start,totalduring,totalcost,PER_MONTH));
		}
		else if (_history_mode==DETAILS)
		{
			for(int32 i=0;i<nbitems;i++)
			{
				itemlist = (HistoryList *)connectionlist->ItemAt(i);
				itemlist->SetNConnect(i+1);
				_historyListView->AddItem(new HistoryList(*itemlist));
			}
		}
	}
}
/***** Remove a connections or connections of a months ****/
void HistoryView::RemoveConnections()
{
	int32		currentselected;
	int32		nbitems;
	tm			tmptime;
	tm			selectedtime;
	time_t		temptime_t;
 	tm			currenttime;
	int32		fixedhours;

	// Get the current time
	temptime_t = time(0);
	currenttime = *localtime(&temptime_t);
	fixedhours = _app->HoursUsed();
	
	_app->FixedMonth(currenttime,false);
	_app->FixedMonth(currenttime,true);
	if(difftime(temptime_t,mktime(&currenttime))<0)
		_app->FixedMonth(currenttime,false);
	
	currentselected = _historyListView->CurrentSelection();
	if(currentselected >= 0)
	{
		selectedtime = ((HistoryList *)_historyListView->ItemAt(currentselected))->Start();
		nbitems = _app->connectionlist->CountItems();
		for(int32 i=0;i<nbitems;i++)
		{
			tmptime = ((HistoryList *)_app->connectionlist->ItemAt(i))->Start();
			switch(_history_mode)
			{
			case DETAILS:
				if (!difftime(mktime(&tmptime),mktime(&selectedtime)))
				{
					_app->connectionlist->RemoveItem(i);
					nbitems--;
					i--;
				}
				break;
			case PER_MONTH:
				if(selectedtime.tm_mon == tmptime.tm_mon)
				{
					if (((HistoryList *)_app->connectionlist->ItemAt(i))->IsFixedPrice() && difftime(mktime(&tmptime),mktime(&currenttime))>=0)
						fixedhours -= ((HistoryList *)_app->connectionlist->ItemAt(i))->During();
					_app->connectionlist->RemoveItem(i);
					nbitems--;
					i--;
				}
			}
		}	
		_historyListView->RemoveItem(currentselected);
		_app->SetHoursUsed(fixedhours);

		// refresh N° of connection
		nbitems = _historyListView->CountItems();
		for(int32 i=0;i<nbitems;i++)
			((HistoryList *)_historyListView->ItemAt(i))->SetNConnect(i+1);
		
		// refresh list
		if(_history_mode == PER_MONTH)
		{
			((HistoryWindow *)Window())->ViewDetails()->EraseDatasList();
			((HistoryWindow *)Window())->ViewDetails()->CopyListOfConnections();
		}
		else
		{
			((HistoryWindow *)Window())->ViewPerMonth()->EraseDatasList();
			((HistoryWindow *)Window())->ViewPerMonth()->CopyListOfConnections();
		}
		
		Invalidate(BRect(10,183,285,223));
		(BMessenger(_app)).SendMessage(U_REFRESH_ALL_COUNT);
	}
}

/**** Erase datas on the lst view ****/
void HistoryView::EraseDatasList()
{
	int32	i;
	int32	nbitems;
	
	nbitems = _historyListView->CountItems();
	for(i=0;i<nbitems;i++)
		_historyListView->RemoveItem((int32)0);
}

/**** Add a Connections to the history ****/
void HistoryView::AddNewConnections()
{
 // not at the moment
}