/**********************************/
/* View for the deskbar Replicant */
/**********************************/
#include <stdio.h>
#include <Message.h>
#include <Deskbar.h>
#include <Roster.h>
#include <MenuItem.h>
#include "Counter.h"
#include "ReplicantCounter.h"
#include "Replicant_Bitmap.h"

const char*			A_RC_CLASS			= "class";
const char*			A_RC_CLASS_NAME		= "ReplicantCounter";
const rgb_color		U_REPLICANT_WHITE	= {255,255,255,255};
const rgb_color		U_REPLICANT_GRAY	= {99,99,99,99};


/**** constructor ****/
ReplicantCounter::ReplicantCounter(BRect frame, const char *name)
:BView(frame, name, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_NAVIGABLE | B_PULSE_NEEDED)
{
}

/**** constructor with message ****/
ReplicantCounter::ReplicantCounter(BMessage *message)
: BView(message)
{
}

/**** Init datas ****/
void ReplicantCounter::InitDatas()
{
	_connectToApp = false;
	
	_heure = NULL;
	_popup = NULL;
	_helpTime = NULL;

	SetFontSize(8.0);
	SetViewColor(222,222,222,255);

	_duringtime = 0;
	_totaltime = 0;

	_valueDisplay = &_duringtime;

	_heure = new BBitmap(BRect(0,0,10,8),B_RGB32);
	_heure->SetBits((void *)Relpicant_Bitmap,297,0,B_RGB32);
}

/**** Init datas ****/
void ReplicantCounter::BuildMenu(BMessage *message)
{
	SettingsFile	langageData(CPREF_LANGAGE_FILE,CPREF_PATH);
	int32			langage;
	BString			tempString;
	
	if(message->FindInt32("type-langage",&langage) != B_OK)
		langage = 0;

	// load definition langage
	langageData.Load();
	
	_popup = new BPopUpMenu("replicant-popup");
	_popup->SetFontSize(10.0);
	langageData.FindString("replicant-total",langage,&_totalTime);
	langageData.FindString("replicant-during",langage,&_duringTime);
	_popup->AddItem(new BMenuItem(_totalTime.String(),new BMessage(R_TOTAL_DURING_TIME_MSG)));
	langageData.FindString("replicant-show",langage,&tempString);
	_popup->AddItem(new BMenuItem(tempString.String(),new BMessage(R_WINDOW_REQUESTED_MSG)));
	_popup->AddSeparatorItem();
	langageData.FindString("about",langage,&tempString);
	_popup->AddItem(new BMenuItem(tempString.String(),new BMessage(R_ABOUT_REQUESTED_MSG)));
	langageData.FindString("button-remove",langage,&tempString);
	_popup->AddItem(new BMenuItem(tempString.String(),new BMessage(R_REMOVE_REPLICANT_MSG)));
	(_popup->ItemAt(4))->SetEnabled(false);
}

/**** destructor ****/
ReplicantCounter::~ReplicantCounter()
{
}

/**** archiving ****/
ReplicantCounter *ReplicantCounter::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data,A_RC_CLASS_NAME))
		return NULL;
	return (new ReplicantCounter(data));
}

status_t ReplicantCounter::Archive(BMessage *data, bool deep = true) const
{
	if(BView::Archive(data, deep) == B_OK)
	data->AddString("add_on",APP_SIGNATURE);
	data->AddString(A_RC_CLASS,A_RC_CLASS_NAME);

	return B_OK;
}

/**** attached to window ****/
void ReplicantCounter::AttachedToWindow(void)
{
	// initialize datas
	InitDatas();

	// try to connect to app
	ConnectToApp();
	
	// create bubble help
	_helpTime = new BubbleHelper();
	
	BView::AttachedToWindow();
}

/**** detach from window ****/
void ReplicantCounter::DetachedFromWindow(void)
{
	delete _heure;
	delete _popup;
	delete _helpTime;
}

/**** Message received ****/
void ReplicantCounter::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		// ok connection is good with app
		case R_CONNECT_TO_APP_MSG:
			{
				BMessage	msgToApp(R_PULSE_DEFINE_MSG);
				int32		pulseRate;
		
				_connectToApp = true;
				pulseRate = (Window())->PulseRate();
   				msgToApp.AddInt32("replicant-pulse-rate",pulseRate);
				(BMessenger(APP_SIGNATURE)).SendMessage(&msgToApp);		
			}
			break;
   		case R_PULSE_DEFINE_MSG:
   			{
   				int32	pulseRate;
   				
		   		if(message->FindInt32("pulse-rate",&pulseRate) == B_OK)
   					(Window())->SetPulseRate(pulseRate);
			}
			break;
   		case R_BUILD_MENU_MSG:
			BuildMenu(message);
			break;
   		case R_UPDATE_TOTAL_TIME_MSG:
   			{
		   		if(message->FindInt32("total-time",&_totaltime) == B_OK)
				{
				UpdateBubbleHelp();
				_helpTime->EnableHelp();
   				Invalidate();
   				}
   			}
   			break;
		/* receive message from app to start connection */
		case R_START_CONNECTION_MSG:
			StartConnection(message);
			break;
		/* receive message from app to end connection */
		case R_END_CONNECTION_MSG:
			EndConnection();
			break;
		/* ask to show the netcount window */
		case R_WINDOW_REQUESTED_MSG:
			(BMessenger(APP_SIGNATURE)).SendMessage(message);
			break;
		/* ask to show the about window */
		case R_ABOUT_REQUESTED_MSG:
			(BMessenger(APP_SIGNATURE)).SendMessage(message);
			break;
		/* ask to show during/total time */
		case R_TOTAL_DURING_TIME_MSG:
			ChooseDisplayTime();
			break;
		/* remove replicant */
		case R_REMOVE_REPLICANT_MSG:
			{
	    	BDeskbar desk;

    		LockLooper();
    		desk.RemoveItem("ReplicantCounter");
    		UnlockLooper();
			}
			break;
		default:
			BView::MessageReceived(message);
	}	
}

/**** mouse down : display popup ****/
void ReplicantCounter::MouseDown(BPoint point)
{
	uint32 		Button = 0;
	BMenuItem	*select;
	
	GetMouse(&point, &Button);
	
	switch(Button) 
	{
	case B_SECONDARY_MOUSE_BUTTON:
		{
		BRoster		aRoster;
		app_info	netCount_info;

		(_popup->ItemAt(4))->SetEnabled(!(aRoster.GetAppInfo(APP_SIGNATURE,&netCount_info) == B_OK));
		
		ConvertToScreen(&point);
		select = _popup->Go(point);
		if (select)
			_messenger.SendMessage(select->Message());
		}
		break;
	case B_PRIMARY_MOUSE_BUTTON:
		ChooseDisplayTime();
		break;
	default:
		BView::MouseDown(point);
	}	
}

/**** draw ****/
void ReplicantCounter::Draw(BRect updateRect)
{
	static	char tempstringdraw[30];
	BRect		frame;
	int32		h,m,s;
	rgb_color	oldColor;
	rgb_color	oldLowColor;

	oldColor = HighColor();
	oldLowColor = LowColor();
	SetLowColor(ViewColor());
	frame = Bounds();

	DrawBitmap(_heure,BPoint(1,3));

	// draw rectangle
	BeginLineArray(4);
	AddLine(BPoint(frame.right,frame.top),BPoint(frame.right,frame.bottom),U_REPLICANT_GRAY);
	AddLine(BPoint(frame.left,frame.bottom),BPoint(frame.right,frame.bottom),U_REPLICANT_GRAY);
	AddLine(BPoint(frame.left,frame.top),BPoint(frame.right,frame.top),U_REPLICANT_WHITE);
	AddLine(BPoint(frame.left,frame.top),BPoint(frame.left,frame.bottom),U_REPLICANT_WHITE);
	EndLineArray();

	h = *_valueDisplay / 3600;
	m = (*_valueDisplay % 3600) / 60;
	s = *_valueDisplay % 60;
	
	sprintf(tempstringdraw,"%02ld:%02ld:%02ld",h,m,s);
	DrawString(tempstringdraw,BPoint(14,11));

	SetHighColor(oldColor);
	SetLowColor(oldLowColor);
}

void ReplicantCounter::Pulse()
{
	// Update counter when the PPP connection is on
	if (_connected == true)
	{
		time_t		temptime;
 		tm			currenttime;
		int32		oldduringtime;
			
		// Get the current time
		temptime = time(0);
		currenttime = *localtime(&temptime);

		// Update surf time string
		oldduringtime = _duringtime;
		_duringtime = (int32)difftime(mktime(&currenttime),mktime(&_starttime));
		
		if (_duringtime != oldduringtime)
			if(_valueDisplay == &_duringtime)
				Invalidate(BRect(14,5,46,11));
			else
				UpdateBubbleHelp();
	}
	else if (!_connectToApp)
		ConnectToApp();

}

void ReplicantCounter::ChooseDisplayTime()
{
	if(_valueDisplay == &_totaltime)
	{
		_valueDisplay = &_duringtime;
		(_popup->ItemAt(0))->SetLabel(_totalTime.String());
	}
	else
	{
		_valueDisplay = &_totaltime;
		(_popup->ItemAt(0))->SetLabel(_duringTime.String());	
	}

	UpdateBubbleHelp();
	Invalidate(BRect(14,5,46,11));
}

/**** End Connection ****/
void ReplicantCounter::EndConnection()
{
	time_t		temptime;
 	tm			currenttime;

	_connected = false;

	// Get the current time
	temptime = time(0);
	currenttime = *localtime(&temptime);

	// Update surf time
	_duringtime = (int32)difftime(mktime(&currenttime),mktime(&_starttime));

	Invalidate(BRect(14,5,46,11));
}

/* StartConnection() */
void ReplicantCounter::StartConnection(BMessage *message)
{
	tm		*start;
	ssize_t	tailleobjet;

	_duringtime = 0;
	message->FindData("start-time",B_ANY_TYPE,(const void **)&start,&tailleobjet);
	_starttime = *start;

	_connected = true;
}

/* StartConnection() */
void ReplicantCounter::UpdateBubbleHelp()
{
	int32			h,m,s;
	int32			bubbleTime;
	static char		bubbleText[30];

	LockLooper();
	if(_valueDisplay == &_totaltime)
		bubbleTime = _duringtime;
	else
		bubbleTime = _totaltime;

	h = bubbleTime / 3600;
	m = (bubbleTime % 3600) / 60;
	s = bubbleTime % 60;
	
	sprintf(bubbleText,"%s : %02ld:%02ld:%02ld",(_popup->ItemAt(0))->Label(),h,m,s);
	_helpTime->SetHelp(this,bubbleText);
	UnlockLooper();
}

/* Connect Messenging To App */
void ReplicantCounter::ConnectToApp()
{
	BMessage	msgToApp(R_CONNECT_TO_APP_MSG);
	
	_messenger = BMessenger(this);
   	msgToApp.AddMessenger("replicant-messenger",_messenger);   	
	(BMessenger(APP_SIGNATURE)).SendMessage(&msgToApp);
}