/*************************************/
/* Vues  des preferences application */
/*************************************/
#include <Control.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include "PrefAllViews.h"
#include "Counter.h"

/**************************************************/
/* vue de reglage des parametres de l'application */
/**************************************************/
ApplicationSettingView::ApplicationSettingView(BRect frame)
: BView(frame,((CounterApp *)be_app)->Langage("app-pref"),B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	SetViewColor(219,219,219,255);
	_app = (CounterApp *)be_app;

	_applicationBox = new BBox(BRect(10,8,220,132),"appli-box");
	_applicationBox->SetLabel("Application");
	AddChild(_applicationBox);
	
	// check bor for options
	_viewDeskbar = new BCheckBox(BRect(6,15,150,30),"deskbar",_app->Langage("app-deskbar"),new BMessage(U_VIEW_DESKBAR_MSG));
	_minimizeAtStart = new BCheckBox(BRect(6,35,200,50),"minimize",_app->Langage("app-minimize"),new BMessage(U_MINIMIZE_AT_START_MSG));
	_displayType = new BCheckBox(BRect(6,55,200,70),"display-type",_app->Langage("app-display"),new BMessage(U_DISPLAY_TYPE_MSG));
	_pulseRate = new BCheckBox(BRect(6,75,200,90),"pulse-rate",_app->Langage("app-rate"),new BMessage(U_PULSE_RATE_MSG));
	_saveAtEnd = new BCheckBox(BRect(6,95,200,110),"save-at-end",_app->Langage("app-save"),new BMessage(U_SAVE_END_MSG));

	_applicationBox->AddChild(_viewDeskbar);
	_applicationBox->AddChild(_minimizeAtStart);
	_applicationBox->AddChild(_displayType);
	_applicationBox->AddChild(_pulseRate);
	_applicationBox->AddChild(_saveAtEnd);
}

ApplicationSettingView::~ApplicationSettingView(void)
{
}

void ApplicationSettingView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case U_SELECT_PTAB_MSG:
		Refresh();
		break;
	case U_VIEW_DESKBAR_MSG:
		{
		_app->SetViewDeskbar(_viewDeskbar->Value());
		if(_viewDeskbar->Value() == B_CONTROL_ON)
			_app->AddDeskbarView();
		else
			_app->RemoveDeskbarView();
		}		
		break;
	case U_MINIMIZE_AT_START_MSG:
		_app->SetMinimizeAtStart(_minimizeAtStart->Value());
		break;
	case U_DISPLAY_TYPE_MSG:
		_app->SetDisplayType(_displayType->Value());
		(BMessenger(_app)).SendMessage(U_REFRESH_ALL_COUNT);
		break;
	case U_PULSE_RATE_MSG:
		_app->SetReplicantRate(_pulseRate->Value());
		break;
	case U_SAVE_END_MSG:
		_app->SetSaveAtEnd(_saveAtEnd->Value());
		break;
	default:
		BView::MessageReceived(msg);
	}
}

void ApplicationSettingView::AttachedToWindow(void)
{
	_viewDeskbar->SetTarget(this);
	_minimizeAtStart->SetTarget(this);
	_displayType->SetTarget(this);
	_pulseRate->SetTarget(this);
	_saveAtEnd->SetTarget(this);

	BView::AttachedToWindow();
}

void ApplicationSettingView::Refresh()
{
	_viewDeskbar->SetValue(_app->ViewDeskbar());
	_minimizeAtStart->SetValue(_app->MinimizeAtStart());
	_displayType->SetValue(_app->DisplayType());
	_pulseRate->SetValue(_app->ReplicantRate());
	_saveAtEnd->SetValue(_app->SaveAtEnd());
}

/*******************************************/
/* vue de reglage des parametres graphique */
/*******************************************/
DisplaySettingView::DisplaySettingView(BRect frame)
: BView(frame,((CounterApp *)be_app)->Langage("display-pref"),B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	rgb_color	tempColor;
	SetViewColor(219,219,219,255);
	_app = (CounterApp *)be_app;

	_exemple = new BTextView(BRect(20,8,210,48),"exemple", BRect(0,0,180, 40), B_FOLLOW_ALL, B_WILL_DRAW);
	_exemple->MakeSelectable(false);
	_exemple->MakeEditable(false);

	_colorsBox = new BBox(BRect(10,55,220,112),"colors-box");
	_colorsBox->SetLabel(_app->Langage("color"));
	AddChild(_colorsBox);

	_colorText = new CColorLabel(BRect(10,13,200,28),"text-color",_app->Langage("display-text"),new BMessage(U_CHNANGE_TEXT_MSG));
	_colorText->SetColor(_app->ColorText());
	_colorView = new CColorLabel(BRect(10,33,200,48),"back-color",_app->Langage("display-back"),new BMessage(U_CHNANGE_BACK_MSG));
	_colorView->SetColor(_app->ColorView());

	// assign color to textview
	tempColor = _app->ColorText();
	_exemple->SetFontAndColor(NULL,B_FONT_ALL,&tempColor);
	_exemple->SetViewColor(_app->ColorView());
	
	// datas for textview
	_exemple->SetText(_app->Langage("display-exem"));
	
	AddChild(_exemple);
	_colorsBox->AddChild(_colorText);
	_colorsBox->AddChild(_colorView);

	// choose your langage
	_langageBox = new BBox(BRect(10,117,220,207),"langage-box");
	_langageBox->SetLabel("Langage");

	_englichRadio = new BRadioButton(BRect(15,20,105,30),"englich-radio",_app->Langage("display-eng"),new BMessage(U_ENGLICH_RADIO_MSG));
	_frenchRadio = new BRadioButton(BRect(15,40,105,50),"french-radio",_app->Langage("display-fre"),new BMessage(U_FRENCH_RADIO_MSG));
	_germanRadio = new BRadioButton(BRect(15,60,105,70),"german-radio",_app->Langage("display-ger"),new BMessage(U_GERMAN_RADIO_MSG));
	_italienRadio = new BRadioButton(BRect(115,20,200,30),"italien-radio",_app->Langage("display-ita"),new BMessage(U_ITALIEN_RADIO_MSG));

	AddChild(_langageBox);
	_langageBox->AddChild(_englichRadio);
	_langageBox->AddChild(_frenchRadio);
	_langageBox->AddChild(_germanRadio);
	_langageBox->AddChild(_italienRadio);
}

DisplaySettingView::~DisplaySettingView(void)
{
}

void DisplaySettingView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case U_SELECT_PTAB_MSG:
		Refresh();
		break;
	case U_CHNANGE_BACK_MSG:
		{
		rgb_color	tempColor = _colorView->Color();
		_app->SetColorView(tempColor);
		_exemple->SetViewColor(tempColor);
		_exemple->Invalidate();
		}
		break;
	case U_CHNANGE_TEXT_MSG:
		{
		rgb_color	tempColor = _colorText->Color();
		_app->SetColorText(tempColor);
		_exemple->SetFontAndColor(NULL,B_FONT_ALL,&tempColor);
		}
		break;
	case U_ENGLICH_RADIO_MSG:
		_app->SetLangage(0);
		break;
	case U_FRENCH_RADIO_MSG:
		_app->SetLangage(1);
		break;
	case U_GERMAN_RADIO_MSG:
		_app->SetLangage(2);
		break;
	case U_ITALIEN_RADIO_MSG:
		_app->SetLangage(3);
		break;
	default:
		BView::MessageReceived(msg);
	}
}

void DisplaySettingView::AttachedToWindow(void)
{
	_colorText->SetTarget(this);
	_colorView->SetTarget(this);
	_englichRadio->SetTarget(this);
	_frenchRadio->SetTarget(this);
	_germanRadio->SetTarget(this);
	_italienRadio->SetTarget(this);

	BView::AttachedToWindow();
}

void DisplaySettingView::Refresh()
{
	rgb_color	tempColor = _app->ColorView();

	_exemple->SetViewColor(tempColor);
	_colorView->SetColor(tempColor);
	_exemple->Invalidate();
	_colorView->Invalidate();
	switch(_app->Langage())
	{
	case 1:
		_frenchRadio->SetValue(B_CONTROL_ON);
		break;
	case 2:
		_germanRadio->SetValue(B_CONTROL_ON);
		break;
	case 3:
		_italienRadio->SetValue(B_CONTROL_ON);
		break;
	default:
		_englichRadio->SetValue(B_CONTROL_ON);
	}
}

/**********************************/
/* vue de reglage de l'historique */
/**********************************/
HistorySettingView::HistorySettingView(BRect frame)
: BView(frame,((CounterApp *)be_app)->Langage("hist-pref"),B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	SetViewColor(219,219,219,255);
	_app = (CounterApp *)be_app;

	_buttonsBox = new BBox(BRect(10,8,220,68),"buttons-box");
	_buttonsBox->SetLabel(_app->Langage("hist-but"));
	AddChild(_buttonsBox);

	_addCheck = new BCheckBox(BRect(10,15,200,30),"add",_app->Langage("hist-add"),new BMessage(U_ADD_CHECK_MSG));
_addCheck->SetEnabled(false);
	_removeCheck = new BCheckBox(BRect(10,35,200,55),"remove",_app->Langage("hist-rem"),new BMessage(U_REMOVE_CHECK_MSG));

	_colorsBox = new BBox(BRect(10,75,220,190),"color-box");
	_colorsBox->SetLabel(_app->Langage("color"));
	AddChild(_colorsBox);

	_colorStandard = new CColorLabel(BRect(10,15,190,30),"standard-color",_app->Langage("hist-std"),new BMessage(U_CHNANGE_STANDARD_MSG));
	_colorFixed = new CColorLabel(BRect(10,34,190,49),"fixed-color",_app->Langage("hist-fpr"),new BMessage(U_CHNANGE_FIXED_MSG));
	_colorDate = new CColorLabel(BRect(10,53,190,68),"date-color",_app->Langage("hist-dat"),new BMessage(U_CHNANGE_DATE_MSG));
	_colorTime = new CColorLabel(BRect(10,72,190,87),"time-color",_app->Langage("hist-tim"),new BMessage(U_CHNANGE_TIME_MSG));
	_colorPrice = new CColorLabel(BRect(10,91,190,106),"price-color",_app->Langage("hist-pri"),new BMessage(U_CHNANGE_PRICE_MSG));

	_buttonsBox->AddChild(_addCheck);
	_buttonsBox->AddChild(_removeCheck);
	_colorsBox->AddChild(_colorStandard);
	_colorsBox->AddChild(_colorFixed);
	_colorsBox->AddChild(_colorDate);
	_colorsBox->AddChild(_colorTime);
	_colorsBox->AddChild(_colorPrice);
}

HistorySettingView::~HistorySettingView(void)
{

}

void HistorySettingView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case U_SELECT_PTAB_MSG:
		Refresh();
		break;
	case U_ADD_CHECK_MSG:
		_app->SetAddState(_addCheck->Value());
		break;
	case U_REMOVE_CHECK_MSG:
		_app->SetRemoveState(_removeCheck->Value());
		break;
	case U_CHNANGE_FIXED_MSG:
		_app->SetColorFixed(_colorFixed->Color());
		break;
	case U_CHNANGE_STANDARD_MSG:
		_app->SetColorStandard(_colorStandard->Color());
		break;
	case U_CHNANGE_DATE_MSG:
		_app->SetColorDate(_colorDate->Color());
		break;
	case U_CHNANGE_TIME_MSG:
		_app->SetColorTime(_colorTime->Color());
		break;
	case U_CHNANGE_PRICE_MSG:
		_app->SetColorPrice(_colorPrice->Color());
		break;
	default:
		BView::MessageReceived(msg);
	}
}

void HistorySettingView::AttachedToWindow(void)
{
	_addCheck->SetTarget(this);
	_removeCheck->SetTarget(this);
	_colorStandard->SetTarget(this);
	_colorFixed->SetTarget(this);
	_colorDate->SetTarget(this);
	_colorTime->SetTarget(this);
	_colorPrice->SetTarget(this);

	BView::AttachedToWindow();
}

void HistorySettingView::Refresh()
{
	_addCheck->SetValue(_app->AddState());
	_removeCheck->SetValue(_app->RemoveState());
	_colorStandard->SetColor(_app->ColorStandard());
	_colorFixed->SetColor(_app->ColorFixed());
	_colorDate->SetColor(_app->ColorDate());
	_colorTime->SetColor(_app->ColorTime());
	_colorPrice->SetColor(_app->ColorPrice());
}

/*****************************/
/* vue de reglage de la v0.6 */
/*****************************/
OldSettingView::OldSettingView(BRect frame)
: BView(frame,((CounterApp *)be_app)->Langage("old-pref"),B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	// delimiter
	BPopUpMenu		*MenuDel;

	SetViewColor(219,219,219,255);
	_app = (CounterApp *)be_app;

	_optionsBox = new BBox(BRect(10,8,220,94),"options-box");
	_optionsBox->SetLabel("Option");
	AddChild(_optionsBox);

	_eraseConnections = new BCheckBox(BRect(10,15,200,30),"erase",_app->Langage("old-erase"),new BMessage(U_ERASE_IMPORT_MSG));
	_recalculTime = new BCheckBox(BRect(10,35,200,50),"recalcul",_app->Langage("old-calcul"),new BMessage(U_RECALCUL_TIME_MSG));
	_formatExport = new BCheckBox(BRect(10,55,200,70),"format",_app->Langage("old-format"),new BMessage(U_FORMAT_VERSION_MSG));

	_optionsBox->AddChild(_eraseConnections);
	_optionsBox->AddChild(_recalculTime);
	_optionsBox->AddChild(_formatExport);

	_impExpBox = new BBox(BRect(10,105,220,210),"impexp-box");
	_impExpBox->SetLabel("Import/Export");
	AddChild(_impExpBox);


	switch(_app->Delimiter())
	{
	case 1:
		MenuDel = new BPopUpMenu(_app->Langage("old-com"));
		break;
	case 2:
		MenuDel = new BPopUpMenu(_app->Langage("old-tab"));
		break;
	default:
		MenuDel = new BPopUpMenu(_app->Langage("old-def"));
	}
	MenuDel->AddItem(new BMenuItem(_app->Langage("old-def"),new BMessage(U_MENU_DEFAULT)));
	MenuDel->AddItem(new BMenuItem(_app->Langage("old-com"),new BMessage(U_MENU_COMMA)));
	MenuDel->AddItem(new BMenuItem(_app->Langage("old-tab"),new BMessage(U_MENU_TABULATION)));
	_delimiter = new BMenuField(BRect(12,17,202,40),"list-delimiter",_app->Langage("old-dti"),MenuDel);

	// path
	_pathinfo =	new BTextControl(BRect(7,43,198,66),"","","",NULL);
	_pathinfo->SetDivider(0);
	_pathinfo->SetEnabled(false);
	// import / export
	_import = new BButton(BRect(10,70,75,90),"import","import",new BMessage(U_IMPORT_MSG));
	_export = new BButton(BRect(133,70,198,90),"export","export",new BMessage(U_EXPORT_MSG));

	_impExpBox->AddChild(_delimiter);
	_impExpBox->AddChild(_import);
	_impExpBox->AddChild(_export);
	_impExpBox->AddChild(_pathinfo);

	// initialise le selecteur de fichier
	BMessenger	oldSettingViewMessenger(this);
	_panelImport = new BFilePanel(B_OPEN_PANEL,&oldSettingViewMessenger,NULL,B_FILE_NODE,false,new BMessage(U_PANEL_IMPORT_MSG),NULL,true);
	_panelExport = new BFilePanel(B_SAVE_PANEL,&oldSettingViewMessenger,NULL,B_FILE_NODE,false,new BMessage(U_PANEL_EXPORT_MSG),NULL,true);
}

OldSettingView::~OldSettingView(void)
{
	delete _panelImport;
	delete _panelExport;
}

void OldSettingView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case U_SELECT_PTAB_MSG:
		Refresh();
		break;
	case U_ERASE_IMPORT_MSG:
		_app->SetEraseState(_eraseConnections->Value());
		break;
	case U_RECALCUL_TIME_MSG:
		_app->SetRecalculTime(_recalculTime->Value());
		break;
	case U_FORMAT_VERSION_MSG:
		_app->SetExportState(_formatExport->Value());
		break;
	case U_IMPORT_MSG:
		_panelImport->SetPanelDirectory(_app->PathImport());
		(_panelImport->Window())->SetTitle(_app->Langage("old-imp"));
		_panelImport->Show();
		break;
	case U_EXPORT_MSG:
		_panelExport->SetPanelDirectory(_app->PathExport());
		(_panelExport->Window())->SetTitle(_app->Langage("old-exp"));
		_panelExport->Show();
		break;
	case U_PANEL_IMPORT_MSG:
		{
			entry_ref	fileRef;

			if ((msg->FindRef("refs",&fileRef)) == B_OK);
			{
				char		name[B_FILE_NAME_LENGTH];
				BEntry		fileEntry(&fileRef,true);
			
				fileEntry.GetName(name);
				_pathinfo->SetText(name);
				_app->ImportFile(&fileEntry);
			}
		}
		break;
	case U_PANEL_EXPORT_MSG:
		{
			entry_ref	pathRef;

			if ((msg->FindRef("directory",&pathRef)) == B_OK);
			{
				BString		name;
				BEntry		pathEntry(&pathRef,true);
			
				msg->FindString("name",&name);
				_pathinfo->SetText(name.String());
				_app->ExportFile(&pathEntry,&name,0,_app->connectionlist->CountItems());
			}
		}
		break;
	case U_MENU_DEFAULT:
		_app->SetDelimiter(0);
		break;
	case U_MENU_COMMA:
		_app->SetDelimiter(1);
		break;
	case U_MENU_TABULATION:
		_app->SetDelimiter(2);
		break;
	default:
		BView::MessageReceived(msg);
	}
}

void OldSettingView::AttachedToWindow(void)
{
	_eraseConnections->SetTarget(this);
	_recalculTime->SetTarget(this);
	_formatExport->SetTarget(this);
	_import->SetTarget(this);
	_export->SetTarget(this);
	// filePanel
	_panelImport->SetTarget(this);
	_panelExport->SetTarget(this);
	(_delimiter->Menu())->SetTargetForItems(this);

	BView::AttachedToWindow();
}

void OldSettingView::Refresh()
{
	_eraseConnections->SetValue(_app->EraseState());
	_formatExport->SetValue(_app->ExportState());
	_recalculTime->SetValue(_app->RecalculTime());
}

/****************************************/
/* vue de reglage des options du backup */
/****************************************/
BackupSettingView::BackupSettingView(BRect frame)
: BView(frame,((CounterApp *)be_app)->Langage("backup-pref"),B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	// delay
	BPopUpMenu		*MenuDel;

	SetViewColor(219,219,219,255);
	_app = (CounterApp *)be_app;

	_optionsBox = new BBox(BRect(10,8,220,114),"options-box");
	_optionsBox->SetLabel(_app->Langage("bcp-backup"));
	AddChild(_optionsBox);

	_useBackup = new BCheckBox(BRect(10,17,200,32),"use-backup",_app->Langage("bcp-use"),new BMessage(U_USE_BACKUP_MSG));

	switch(_app->DelayBackup())
	{
	case 1:
		MenuDel = new BPopUpMenu(_app->Langage("bcp-7day"));
		break;
	case 2:
		MenuDel = new BPopUpMenu(_app->Langage("bcp-14day"));
		break;
	case 4:
		MenuDel = new BPopUpMenu(_app->Langage("bcp-28day"));
		break;
	default:
		MenuDel = new BPopUpMenu(_app->Langage("bcp-day"));
	}
	MenuDel->AddItem(new BMenuItem(_app->Langage("bcp-28day"),new BMessage(U_MENU_28DAYS)));
	MenuDel->AddItem(new BMenuItem(_app->Langage("bcp-14day"),new BMessage(U_MENU_14DAYS)));
	MenuDel->AddItem(new BMenuItem(_app->Langage("bcp-7day"),new BMessage(U_MENU_7DAYS)));
	MenuDel->AddItem(new BMenuItem(_app->Langage("bcp-day"),new BMessage(U_MENU_DAYS)));
	_delay = new BMenuField(BRect(12,40,202,62),"list-delimiter",_app->Langage("bcp-del"),MenuDel);

	_optionsBox->AddChild(_useBackup);
	_optionsBox->AddChild(_delay);
}

BackupSettingView::~BackupSettingView(void)
{
}

void BackupSettingView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case U_SELECT_PTAB_MSG:
		Refresh();
		break;
	case U_USE_BACKUP_MSG:
		_app->SetUseBackup(_useBackup->Value());
		Refresh();
		break;
	case U_MENU_28DAYS:
		_app->SetDelayBackup(4);
		break;
	case U_MENU_14DAYS:
		_app->SetDelayBackup(2);
		break;
	case U_MENU_7DAYS:
		_app->SetDelayBackup(1);
		break;
	case U_MENU_DAYS:
		_app->SetDelayBackup(0);
		break;
	default:
		BView::MessageReceived(msg);
	}
}

void BackupSettingView::AttachedToWindow(void)
{
	_useBackup->SetTarget(this);
	(_delay->Menu())->SetTargetForItems(this);

	BView::AttachedToWindow();
}

void BackupSettingView::Refresh()
{
	_useBackup->SetValue(_app->UseBackup());
	_delay->SetEnabled(_useBackup->Value() == B_CONTROL_ON);
}
